@@ -19,11 +19,24 @@
 
 #include "appdata.h"
 
+#ifdef ENABLE_OSM_GPS_MAP
+#include "osm-gps-map.h"
+#endif
+
+#define TAB_LABEL_MAP    "Map"
+#define TAB_LABEL_DIRECT "Direct"
+#define TAB_LABEL_EXTENT "Extent"
+#define TAB_LABEL_MM     "Maemo Mapper"
+
+/* limit of square kilometers above the warning is enabled */
+#define WARN_OVER  5.0
+
 typedef struct {
   GtkWidget *dialog, *notebook;
   area_edit_t *area;
   pos_t min, max;      /* local copy to work on */
   GtkWidget *minlat, *maxlat, *minlon, *maxlon;
+  GtkWidget *warning;
 
   struct {
     GtkWidget *minlat, *maxlat, *minlon, *maxlon;
@@ -34,28 +47,128 @@ typedef struct {
     gboolean is_mil;
   } extent;
 
+#ifdef USE_HILDON 
   struct {
     GtkWidget *fetch;
   } mmapper;
+#endif
+
+  struct {
+    GtkWidget *widget;
+    GtkWidget *zoomin, *zoomout;
+    gboolean needs_redraw;
+  } map;
 
 } context_t;
 
-static void parse_and_set_lat(GtkWidget *src, GtkWidget *dst, pos_float_t *store) {
+static void parse_and_set_lat(GtkWidget *src, pos_float_t *store) {
   pos_float_t i = pos_parse_lat((char*)gtk_entry_get_text(GTK_ENTRY(src)));
-  if(pos_lat_valid(i)) {
+  if(pos_lat_valid(i)) 
     *store = i;
-    pos_lat_label_set(dst, i);
-  }
 }
 
-static void parse_and_set_lon(GtkWidget *src, GtkWidget *dst, pos_float_t *store) {
+static void parse_and_set_lon(GtkWidget *src, pos_float_t *store) {
   pos_float_t i = pos_parse_lon((char*)gtk_entry_get_text(GTK_ENTRY(src)));
-  if(pos_lon_valid(i)) {
+  if(pos_lon_valid(i)) 
     *store = i;
-    pos_lon_label_set(dst, i);
-  }
 }
 
+#define LOG2(x) (log(x) / log(2))
+
+static gboolean current_tab_is(context_t *context, gint page_num, char *str) {
+  if(page_num < 0)
+    page_num = 
+      gtk_notebook_get_current_page(GTK_NOTEBOOK(context->notebook));
+
+  if(page_num < 0) return FALSE;
+
+  GtkWidget *w = 
+    gtk_notebook_get_nth_page(GTK_NOTEBOOK(context->notebook), page_num);
+  const char *name = 
+    gtk_notebook_get_tab_label_text(GTK_NOTEBOOK(context->notebook), w);
+  
+  return(strcasecmp(name, _(str)) == 0);
+}
+
+static void on_area_warning_clicked(GtkButton *button, gpointer data) {
+  context_t *context = (context_t*)data;
+
+  /* compute area size */
+  pos_float_t center_lat = (context->max.lat + context->min.lat)/2;
+  double vscale = DEG2RAD(POS_EQ_RADIUS / 1000.0);
+  double hscale = DEG2RAD(cos(DEG2RAD(center_lat)) * POS_EQ_RADIUS / 1000.0);
+
+  double area = vscale * (context->max.lat - context->min.lat) *
+    hscale * (context->max.lon - context->min.lon);
+
+  warningf(context->dialog,
+   _("The currently selected area is %.02f km² (%.02f mil²) in size. "
+     "This is more than the recommended %.02f km² (%.02f mil²).\n\n"
+     "Continuing may result in a big download and low mapping performance "
+     "in a densly mapped area (e.g. cities)!"), 
+	   area, area/(KMPMIL*KMPMIL),
+	   WARN_OVER, WARN_OVER/(KMPMIL*KMPMIL)
+	   );
+
+}
+
+static void area_main_update(context_t *context) {
+  pos_lat_label_set(context->minlat, context->min.lat);
+  pos_lat_label_set(context->maxlat, context->max.lat);
+  pos_lon_label_set(context->minlon, context->min.lon);
+  pos_lon_label_set(context->maxlon, context->max.lon);
+
+  /* check if area size exceeds recommended values */
+  pos_float_t center_lat = (context->max.lat + context->min.lat)/2;
+  double vscale = DEG2RAD(POS_EQ_RADIUS / 1000.0);
+  double hscale = DEG2RAD(cos(DEG2RAD(center_lat)) * POS_EQ_RADIUS / 1000.0);
+
+  double area = vscale * (context->max.lat - context->min.lat) *
+    hscale * (context->max.lon - context->min.lon);
+
+  if(area > WARN_OVER)
+    gtk_widget_show(context->warning);
+  else
+    gtk_widget_hide(context->warning);
+}
+
+#ifdef ENABLE_OSM_GPS_MAP
+/* the contents of the map tab have been changed */
+static void map_update(context_t *context, gboolean forced) {
+
+  /* map is first tab (page 0) */
+  if(!forced && !current_tab_is(context, -1, TAB_LABEL_MAP)) {
+    context->map.needs_redraw = TRUE;
+    return;
+  }
+
+  pos_float_t center_lat = (context->max.lat + context->min.lat)/2;
+  pos_float_t center_lon = (context->max.lon + context->min.lon)/2;
+
+  /* we know the widgets pixel size, we know the required real size, */
+  /* we want the zoom! */
+  double vzoom = LOG2((45.0 * context->map.widget->allocation.height)/
+		      ((context->max.lat - context->min.lat)*32.0));
+
+  double hzoom = LOG2((45.0 * context->map.widget->allocation.width)/
+		      ((context->max.lon - context->min.lon)*32.0));
+
+  osm_gps_map_set_center(OSM_GPS_MAP(context->map.widget),
+			 center_lat, center_lon);	    
+
+  osm_gps_map_set_zoom(OSM_GPS_MAP(context->map.widget), (hzoom+vzoom+0.5)/2);
+
+  context->map.needs_redraw = FALSE;
+}
+
+static gboolean on_map_configure(GtkWidget *widget,
+				 GdkEventConfigure *event,
+				 context_t *context) {
+  map_update(context, FALSE);
+  return FALSE;
+}
+#endif
+
 /* the contents of the direct tab have been changed */
 static void direct_update(context_t *context) {
   pos_lat_entry_set(context->direct.minlat, context->min.lat);
@@ -85,25 +198,30 @@ static void extent_update(context_t *context) {
 static void callback_modified_direct(GtkWidget *widget, gpointer data) {
   context_t *context = (context_t*)data;
 
-  /* direct is first tab (page 0) */
-  if(gtk_notebook_get_current_page(GTK_NOTEBOOK(context->notebook)) != 0)
+  /* direct is second tab (page 1) */
+  if(!current_tab_is(context, -1, TAB_LABEL_DIRECT)) 
     return;
 
   /* parse the fields from the direct entry pad */
-  parse_and_set_lat(context->direct.minlat, context->minlat, &context->min.lat);
-  parse_and_set_lon(context->direct.minlon, context->minlon, &context->min.lon);
-  parse_and_set_lat(context->direct.maxlat, context->maxlat, &context->max.lat);
-  parse_and_set_lon(context->direct.maxlon, context->maxlon, &context->max.lon);
+  parse_and_set_lat(context->direct.minlat, &context->min.lat);
+  parse_and_set_lon(context->direct.minlon, &context->min.lon);
+  parse_and_set_lat(context->direct.maxlat, &context->max.lat);
+  parse_and_set_lon(context->direct.maxlon, &context->max.lon);
+
+  area_main_update(context);
 
   /* also adjust other views */
   extent_update(context);
+#ifdef ENABLE_OSM_GPS_MAP
+  map_update(context, FALSE);
+#endif
 }
 
 static void callback_modified_extent(GtkWidget *widget, gpointer data) {
   context_t *context = (context_t*)data;
 
-  /* extent is second tab (page 1) */
-  if(gtk_notebook_get_current_page(GTK_NOTEBOOK(context->notebook)) != 1)
+  /* extent is third tab (page 2) */
+  if(!current_tab_is(context, -1, TAB_LABEL_EXTENT)) 
     return;
 
   pos_float_t center_lat = pos_lat_get(context->extent.lat);
@@ -120,18 +238,19 @@ static void callback_modified_extent(GtkWidget *widget, gpointer data) {
 
   height /= 2 * vscale;
   context->min.lat = center_lat - height;
-  pos_lat_label_set(context->minlat, context->min.lat);
   context->max.lat = center_lat + height;
-  pos_lat_label_set(context->maxlat, context->max.lat);
-  
+
   width /= 2 * hscale;
   context->min.lon = center_lon - width;
-  pos_lon_label_set(context->minlon, context->min.lon);
   context->max.lon = center_lon + width;
-  pos_lon_label_set(context->maxlon, context->max.lon);
+
+  area_main_update(context);
   
   /* also update other tabs */
   direct_update(context);
+#ifdef ENABLE_OSM_GPS_MAP
+  map_update(context, FALSE);
+#endif
 }
 
 static void callback_modified_unit(GtkWidget *widget, gpointer data) {
@@ -154,8 +273,6 @@ static void callback_modified_unit(GtkWidget *widget, gpointer data) {
 static void callback_fetch_mm_clicked(GtkButton *button, gpointer data) {
   context_t *context = (context_t*)data;
 
-  printf("clicked fetch mm!\n");
-
   if(!dbus_mm_set_position(context->area->osso_context, NULL)) {
     errorf(context->dialog, 
 	   _("Unable to communicate with Maemo Mapper. "
@@ -173,8 +290,8 @@ static void callback_fetch_mm_clicked(GtkButton *button, gpointer data) {
     return;
   }
 
-  /* maemo mapper is third tab (page 2) */
-  if(gtk_notebook_get_current_page(GTK_NOTEBOOK(context->notebook)) != 2)
+  /* maemo mapper is fourth tab (page 3) */
+  if(!current_tab_is(context, -1, TAB_LABEL_MM)) 
     return;
 
   /* maemo mapper pos data ... */
@@ -188,24 +305,89 @@ static void callback_fetch_mm_clicked(GtkButton *button, gpointer data) {
   double vscale = DEG2RAD(POS_EQ_RADIUS);
   double height = 8 * (1<<zoom) / vscale;
   context->min.lat = center_lat - height;
-  pos_lat_label_set(context->minlat, context->min.lat);
   context->max.lat = center_lat + height;
-  pos_lat_label_set(context->maxlat, context->max.lat);
   
   double hscale = DEG2RAD(cos(DEG2RAD(center_lat)) * POS_EQ_RADIUS);
   double width  = 16 * (1<<zoom) / hscale;
   context->min.lon = center_lon - width;
-  pos_lon_label_set(context->minlon, context->min.lon);
   context->max.lon = center_lon + width;
-  pos_lon_label_set(context->maxlon, context->max.lon);
+
+  area_main_update(context);
 
   /* also update other tabs */
   direct_update(context);
   extent_update(context);
+#ifdef ENABLE_OSM_GPS_MAP
+  map_update(context, FALSE);
+#endif
+}
+#endif
+
+#ifdef ENABLE_OSM_GPS_MAP
+/* the user has changed the map view, update other views accordingly */
+static void map_has_changed(context_t *context) {
+  coord_t pt1, pt2;
+
+  /* get maps bounding box */
+  osm_gps_map_get_bbox(OSM_GPS_MAP(context->map.widget), &pt1, &pt2);
+
+  context->min.lat = RAD2DEG(pt2.rlat);
+  context->max.lat = RAD2DEG(pt1.rlat);
+
+  context->min.lon = RAD2DEG(pt1.rlon);
+  context->max.lon = RAD2DEG(pt2.rlon);
+
+  area_main_update(context);
+  direct_update(context);
+  extent_update(context);
+}
+
+static gboolean
+on_map_button_release_event(GtkWidget *widget, 
+			    GdkEventButton *event, context_t *context) {
+  map_has_changed(context);
+  return FALSE;
+}
+
+static void map_zoom(context_t *context, int step) {
+  int zoom;
+  OsmGpsMap *map = OSM_GPS_MAP(context->map.widget);
+  g_object_get(map, "zoom", &zoom, NULL);
+  zoom = osm_gps_map_set_zoom(map, zoom+step);
+
+  /* enable/disable zoom buttons as required */
+  gtk_widget_set_sensitive(context->map.zoomin, zoom<17);
+  gtk_widget_set_sensitive(context->map.zoomout, zoom>1);
+
+  map_has_changed(context);
+}
+
+static gboolean
+cb_map_zoomin(GtkButton *button, context_t *context) {
+  map_zoom(context, +1);
+  return FALSE;
+}
+
+static gboolean
+cb_map_zoomout(GtkButton *button, context_t *context) {
+  map_zoom(context, -1);
+  return FALSE;
+}
+
+static void on_page_switch(GtkNotebook *notebook, GtkNotebookPage *page,
+			   guint page_num, context_t *context) {
+
+  /* updating the map while the user manually changes some coordinates */
+  /* may confuse the map. so we delay those updates until the map tab */
+  /* is becoming visible */
+  if(current_tab_is(context, page_num, TAB_LABEL_MAP) && 
+     context->map.needs_redraw) 
+    map_update(context, TRUE);
 }
 #endif
 
 gboolean area_edit(area_edit_t *area) {
+  GtkWidget *vbox;
   gboolean ok = FALSE;
 
   context_t context;
@@ -217,59 +399,106 @@ gboolean area_edit(area_edit_t *area) {
   context.max.lon = area->max->lon;
 
   context.dialog = 
-    misc_dialog_new(MISC_DIALOG_NOSIZE, _("Area editor"),
+    misc_dialog_new(MISC_DIALOG_HIGH, _("Area editor"),
 	  GTK_WINDOW(area->parent),
 	  GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, 
           GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
           NULL);
 
-  GtkWidget *table = gtk_table_new(3, 3, FALSE);  // x, y
+  GtkWidget *table = gtk_table_new(5, 2, FALSE);  // x, y
 
-  GtkWidget *label = gtk_label_new(_("Latitude"));
-  gtk_table_attach_defaults(GTK_TABLE(table),  label, 1, 2, 0, 1);
-  label = gtk_label_new(_("Longitude"));
-  gtk_table_attach_defaults(GTK_TABLE(table),  label, 2, 3, 0, 1);
-
-  label = gtk_label_new(_("Min:"));
-  gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
-  gtk_table_attach_defaults(GTK_TABLE(table),  label, 0, 1, 1, 2);
+  GtkWidget *label = gtk_label_new(_("Latitude:"));
+  misc_table_attach(table, label, 0, 0);
   context.minlat = pos_lat_label_new(area->min->lat);
-  gtk_table_attach_defaults(GTK_TABLE(table), context.minlat, 1, 2, 1, 2);
-  context.minlon = pos_lon_label_new(area->min->lon);
-  gtk_table_attach_defaults(GTK_TABLE(table), context.minlon, 2, 3, 1, 2);
-
-  label = gtk_label_new(_("Max:"));
-  gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
-  gtk_table_attach_defaults(GTK_TABLE(table),  label, 0, 1, 2, 3);
+  misc_table_attach(table, context.minlat, 1, 0);
+  label = gtk_label_new(_("to"));
+  misc_table_attach(table, label, 2, 0);
   context.maxlat = pos_lat_label_new(area->max->lat);
-  gtk_table_attach_defaults(GTK_TABLE(table), context.maxlat, 1, 2, 2, 3);
-  context.maxlon = pos_lon_label_new(area->max->lon);
-  gtk_table_attach_defaults(GTK_TABLE(table), context.maxlon, 2, 3, 2, 3);
+  misc_table_attach(table, context.maxlat, 3, 0);
 
-  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(context.dialog)->vbox), 
-			      table);
+  label = gtk_label_new(_("Longitude:"));
+  misc_table_attach(table, label, 0, 1);
+  context.minlon = pos_lon_label_new(area->min->lon);
+  misc_table_attach(table, context.minlon, 1, 1);
+  label = gtk_label_new(_("to"));
+  misc_table_attach(table, label, 2, 1);
+  context.maxlon = pos_lon_label_new(area->max->lon);
+  misc_table_attach(table, context.maxlon, 3, 1);
+
+  context.warning = gtk_button_new();
+  gtk_button_set_image(GTK_BUTTON(context.warning), 
+		       gtk_image_new_from_stock(GTK_STOCK_DIALOG_WARNING, 
+						GTK_ICON_SIZE_BUTTON));
+  g_signal_connect(context.warning, "clicked", 
+  		   G_CALLBACK(on_area_warning_clicked), &context);
+  gtk_table_attach_defaults(GTK_TABLE(table), context.warning, 4, 5, 0, 2);
+
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(context.dialog)->vbox),
+			      table, FALSE, FALSE, 0);
 
   context.notebook = gtk_notebook_new();
 
+#ifdef ENABLE_OSM_GPS_MAP
+  /* ------------- fetch from map ------------------------ */
+
+  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
+
+  context.map.needs_redraw = FALSE;
+  context.map.widget = g_object_new(OSM_TYPE_GPS_MAP,
+	        "repo-uri", MAP_SOURCE_OPENSTREETMAP,
+		"proxy-uri", misc_get_proxy_uri(area->settings),
+		 NULL);
+
+  g_signal_connect(G_OBJECT(context.map.widget), "configure-event",
+		   G_CALLBACK(on_map_configure), &context);
+  g_signal_connect(G_OBJECT(context.map.widget), "button-release-event",
+		   G_CALLBACK(on_map_button_release_event), &context);
+
+  gtk_box_pack_start_defaults(GTK_BOX(hbox), context.map.widget);
+
+  /* zoom button box */
+  vbox = gtk_vbox_new(FALSE,0);
+
+  context.map.zoomin = gtk_button_new();
+  gtk_button_set_image(GTK_BUTTON(context.map.zoomin), 
+       gtk_image_new_from_stock(GTK_STOCK_ZOOM_IN, GTK_ICON_SIZE_MENU));
+  g_signal_connect(context.map.zoomin, "clicked", 
+		   G_CALLBACK(cb_map_zoomin), &context);
+  gtk_box_pack_start(GTK_BOX(vbox), context.map.zoomin, FALSE, FALSE, 0);
+
+  context.map.zoomout = gtk_button_new();
+  gtk_button_set_image(GTK_BUTTON(context.map.zoomout), 
+       gtk_image_new_from_stock(GTK_STOCK_ZOOM_OUT, GTK_ICON_SIZE_MENU));
+  g_signal_connect(context.map.zoomout, "clicked", 
+		   G_CALLBACK(cb_map_zoomout), &context);
+  gtk_box_pack_start(GTK_BOX(vbox), context.map.zoomout, FALSE, FALSE, 0);
+
+  gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
+
+  gtk_notebook_append_page(GTK_NOTEBOOK(context.notebook),
+		   hbox, gtk_label_new(_(TAB_LABEL_MAP)));
+#endif
+
   /* ------------ direct min/max edit --------------- */
 
+  vbox = gtk_vbox_new(FALSE, 10);
   table = gtk_table_new(3, 3, FALSE);  // x, y
+  gtk_table_set_col_spacings(GTK_TABLE(table), 10);
+  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
 
-  label = gtk_label_new(_("Min:"));
-  gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
-  gtk_table_attach_defaults(GTK_TABLE(table),  label, 0, 1, 0, 1);
-  context.direct.minlat = pos_lat_entry_new(0.);
-  gtk_table_attach_defaults(GTK_TABLE(table), context.direct.minlat, 1, 2, 0, 1);
-  context.direct.minlon = pos_lon_entry_new(area->min->lon);
-  gtk_table_attach_defaults(GTK_TABLE(table), context.direct.minlon, 2, 3, 0, 1);
-
-  label = gtk_label_new(_("Max:"));
-  gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
-  gtk_table_attach_defaults(GTK_TABLE(table),  label, 0, 1, 1, 2);
+  context.direct.minlat = pos_lat_entry_new(0.0);
+  misc_table_attach(table, context.direct.minlat, 0, 0);
+  label = gtk_label_new(_("to"));
+  misc_table_attach(table,  label, 1, 0);
   context.direct.maxlat = pos_lat_entry_new(0.0);
-  gtk_table_attach_defaults(GTK_TABLE(table), context.direct.maxlat, 1, 2, 1, 2);
+  misc_table_attach(table, context.direct.maxlat, 2, 0);
+
+  context.direct.minlon = pos_lon_entry_new(area->min->lon);
+  misc_table_attach(table, context.direct.minlon, 0, 1);
+  label = gtk_label_new(_("to"));
+  misc_table_attach(table,  label, 1, 1);
   context.direct.maxlon = pos_lon_entry_new(0.0);
-  gtk_table_attach_defaults(GTK_TABLE(table), context.direct.maxlon, 2, 3, 1, 2);
+  misc_table_attach(table, context.direct.maxlon, 2, 1);
 
   /* setup this page */
   direct_update(&context);
@@ -288,12 +517,16 @@ gboolean area_edit(area_edit_t *area) {
   label = gtk_label_new(_("(recommended min/max diff <0.03 degrees)"));
   gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 3, 2, 3);
 
+  gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
   gtk_notebook_append_page(GTK_NOTEBOOK(context.notebook),
-		 table, gtk_label_new(_("Direct")));
+	   vbox, gtk_label_new(_(TAB_LABEL_DIRECT)));
 
   /* ------------- center/extent edit ------------------------ */
 
+  vbox = gtk_vbox_new(FALSE, 10);
   table = gtk_table_new(3, 4, FALSE);  // x, y
+  gtk_table_set_col_spacings(GTK_TABLE(table), 10);
+  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
 
   label = gtk_label_new(_("Center:"));
   gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
@@ -303,7 +536,7 @@ gboolean area_edit(area_edit_t *area) {
   context.extent.lon = pos_lon_entry_new(0.0);
   gtk_table_attach_defaults(GTK_TABLE(table), context.extent.lon, 2, 3, 0, 1);
 
-  gtk_table_set_row_spacing(GTK_TABLE(table), 0, 8);
+  gtk_table_set_row_spacing(GTK_TABLE(table), 0, 10);
 
   label = gtk_label_new(_("Width:"));
   gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
@@ -344,37 +577,44 @@ gboolean area_edit(area_edit_t *area) {
   label = gtk_label_new(_("(recommended width/height < 2km/1.25mi)"));
   gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 3, 3, 4);
 
-
+  gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
   gtk_notebook_append_page(GTK_NOTEBOOK(context.notebook),
-		   table, gtk_label_new(_("Extent")));
+		   vbox, gtk_label_new(_(TAB_LABEL_EXTENT)));
 
 #ifdef USE_HILDON 
   /* ------------- fetch from maemo mapper ------------------------ */
 
-  GtkWidget *vbox = gtk_vbox_new(FALSE, 8);
+  vbox = gtk_vbox_new(FALSE, 8);
   context.mmapper.fetch = 
     gtk_button_new_with_label(_("Get from Maemo Mapper"));
-  gtk_box_pack_start_defaults(GTK_BOX(vbox), context.mmapper.fetch);
+  gtk_box_pack_start(GTK_BOX(vbox), context.mmapper.fetch, FALSE, FALSE, 0);
 
   g_signal_connect(G_OBJECT(context.mmapper.fetch), "clicked",
 		   G_CALLBACK(callback_fetch_mm_clicked), &context);
-  //  gtk_widget_set_sensitive(context.mmapper.fetch, context.area->mmpos->valid);
 
   /* --- hint --- */
   label = gtk_label_new(_("(recommended MM zoom level < 7)"));
-  gtk_box_pack_start_defaults(GTK_BOX(vbox), label);
+  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
 
   gtk_notebook_append_page(GTK_NOTEBOOK(context.notebook),
-		   vbox, gtk_label_new(_("Maemo Mapper")));
+		   vbox, gtk_label_new(_(TAB_LABEL_MM)));
 #endif
 
+  /* ------------------------------------------------------ */
+
   gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(context.dialog)->vbox), 
 			      context.notebook);
 
+#ifdef ENABLE_OSM_GPS_MAP
+  g_signal_connect(G_OBJECT(context.notebook), "switch-page",
+		   G_CALLBACK(on_page_switch), &context);
+#endif
 
   gtk_widget_show_all(context.dialog);
 
+  area_main_update(&context);
+
   if(GTK_RESPONSE_ACCEPT == gtk_dialog_run(GTK_DIALOG(context.dialog))) {
     /* copy modified values back to given storage */
     area->min->lat = context.min.lat;
