@@ -20,7 +20,7 @@
 #ifndef RELATION_EDIT_H
 #define RELATION_EDIT_H
 
-void relation_add_dialog(appdata_t *appdata, object_t *object);
+void relation_add_dialog(GtkWidget *parent, appdata_t *appdata, object_t *object);
 void relation_list(GtkWidget *parent, appdata_t *appdata, object_t *object);
 void relation_show_members(GtkWidget *parent, relation_t *relation);
 
